import {
  AlertDialog,
  ButtonProps,
  Tooltip,
  useDisclosure,
  useToast,
  Text,
  AlertDialogOverlay,
  AlertDialogContent,
  AlertDialogHeader,
  AlertDialogBody,
  AlertDialogFooter,
  Button,
} from '@chakra-ui/react'
import React from 'react'
import { useQueryClient } from 'react-query'
import { Animation, Pattern } from 'src/models'
import { DeleteAnimationMutation, GetAnimationsQuery } from '../query-builder'

export type DeleteAnimationButtonProps = {
  animation: Animation
  patternId: Pattern['id']
  hasTooltip?: boolean
  children: React.ReactElement<ButtonProps>
}

export const DeleteAnimationButton: React.FC<DeleteAnimationButtonProps> = ({
  animation,
  patternId,
  hasTooltip = false,
  children,
}) => {
  const toast = useToast()
  const queryClient = useQueryClient()
  const [isLoading, setIsLoading] = React.useState(false)
  const { isOpen, onOpen, onClose } = useDisclosure()
  const cancelRef = React.useRef<HTMLButtonElement>(null)

  const deletePatternMutation = DeleteAnimationMutation.useMutation({
    onSuccess: () => {
      setIsLoading(false)
      toast({
        status: 'success',
        position: 'top',
        title: `Se ha eliminado la animacion ${animation.name} satisfactoriamente.`,
      })
      queryClient.invalidateQueries(GetAnimationsQuery.getKey({ patternId }))
    },
    onError: () => {
      toast({
        status: 'error',
        position: 'top',
        title: `Ha ocurrido un error al eliminar la animacion.`,
      })
      onClose()
      setIsLoading(false)
    },
  })

  const handleOnDelete = () => {
    setIsLoading(true)
    deletePatternMutation.mutate({ id: animation.id })
  }

  if (!React.isValidElement(children) || Array.isArray(children)) {
    throw new Error(
      'DeleteAnimationButton child must be a single clickable element'
    )
  }

  const btn = React.cloneElement(children, { onClick: onOpen })

  return (
    <>
      <Tooltip isDisabled={!hasTooltip} label={<Text>Eliminar animacion</Text>}>
        {btn}
      </Tooltip>
      <AlertDialog
        isOpen={isOpen}
        onClose={onClose}
        leastDestructiveRef={cancelRef}
      >
        <AlertDialogOverlay>
          <AlertDialogContent>
            <AlertDialogHeader fontSize="lg" fontWeight="bold">
              Eliminar animacion?
            </AlertDialogHeader>
            <AlertDialogBody>
              <Text>
                Si eliminas la animacion <Text as="b">{animation.name}</Text>{' '}
                todos sus datos asociados seran eliminados.
              </Text>
            </AlertDialogBody>
            <AlertDialogFooter>
              <Button ref={cancelRef} variant="ghost" onClick={onClose}>
                Cancelar
              </Button>
              <Button
                variant="solid"
                onClick={handleOnDelete}
                ml="3"
                isLoading={isLoading}
              >
                Eliminar
              </Button>
            </AlertDialogFooter>
          </AlertDialogContent>
        </AlertDialogOverlay>
      </AlertDialog>
    </>
  )
}
